# -*- coding: utf-8 -*-

__author__ = 'Jace Xu'

from .autoit import AUTO_IT
from .autoit import api, error
from .autoit import Properties
from .autoit import AutoItError
from ctypes.wintypes import *


@api.check(1, "run program failed")
def run(filename, work_dir="", show_flag=Properties.SW_SHOWNORMAL):
    """

    :param filename:
    :param work_dir:
    :param show_flag:
    :return:
    """
    ret = AUTO_IT.AU3_Run(LPCWSTR(filename), LPCWSTR(work_dir),
                          INT(show_flag))
    return ret


@api.check(1, "run program failed")
def run_wait(filename, work_dir="", show_flag=Properties.SW_SHOWNORMAL):
    """

    :param filename:
    :param work_dir:
    :param show_flag:
    :return:
    """
    ret = AUTO_IT.AU3_RunWait(LPCWSTR(filename), LPCWSTR(work_dir),
                              INT(show_flag))
    return ret


def process_close(process):
    """
    Terminates a named process.
    """
    ret = AUTO_IT.AU3_ProcessClose(LPCWSTR(process))
    return ret


def process_exists(process):
    """

    :param process:
    :return:
    """
    ret = AUTO_IT.AU3_ProcessExists(LPCWSTR(process))
    return ret


def process_set_priority(process, priority):
    """
    Changes the priority of a process
    :param process: The name or PID of the process to check.
    :param priority:A flag which determines what priority to set
        0 - Idle/Low
        1 - Below Normal (Not supported on Windows 95/98/ME)
        2 - Normal
        3 - Above Normal (Not supported on Windows 95/98/ME)
        4 - High
        5 - Realtime (Use with caution, may make the system unstable)
    :return:
    """
    ret = AUTO_IT.AU3_ProcessSetPriority(LPCWSTR(process), INT(priority))
    if ret == 0:
        if error() == 1:
            raise AutoItError("set priority failed")
        elif error() == 2:
            raise AutoItError("unsupported priority class be used")
    return ret


@api.check(2, "the process wait timed out")
def process_wait(process, timeout=0):
    """
    Pauses script execution until a given process exists.
    :param process:
    :param timeout:
    :return:
    """
    ret = AUTO_IT.AU3_ProcessWait(LPCWSTR(process), INT(timeout))
    return ret


@api.check(2, "the process wait close timed out")
def process_wait_close(process, timeout=0):
    """
    Pauses script execution until a given process does not exist.
    :param process:
    :param timeout:
    :return:
    """
    ret = AUTO_IT.AU3_ProcessWaitClose(LPCWSTR(process), INT(timeout))
    return ret


@api.check(1, "run an external program failed")
def run_as(user, domain, password, filename, logon_flag=1, work_dir="",
           show_flag=Properties.SW_SHOWNORMAL):
    """
    Runs an external program.
    :param user: username The user name to use.
    :param domain: The domain name to use.
    :param password: The password to use.
    :param logon_flag: 0 = do not load the user profile, 1 = (default) load
        the user profile, 2 = use for net credentials only
    :param filename: The name of the executable (EXE, BAT, COM, or PIF) to run.
    :param work_dir: The working directory.
    :param show_flag: The "show" flag of the executed program:
        SW_HIDE = Hidden window
        SW_MINIMIZE = Minimized window
        SW_MAXIMIZE = Maximized window
    :return:
    """
    ret = AUTO_IT.AU3_RunAs(
        LPCWSTR(user), LPCWSTR(domain), LPCWSTR(password), INT(logon_flag),
        LPCWSTR(filename), LPCWSTR(work_dir), INT(show_flag)
    )
    return ret


@api.check(1, "run an external program failed")
def run_as_wait(user, domain, password, filename, logon_flag=1, work_dir="",
                show_flag=Properties.SW_SHOWNORMAL):
    """
    Runs an external program.
    :param user: username The user name to use.
    :param domain: The domain name to use.
    :param password: The password to use.
    :param logon_flag: 0 = do not load the user profile, 1 = (default) load
        the user profile, 2 = use for net credentials only
    :param filename: The name of the executable (EXE, BAT, COM, or PIF) to run.
    :param work_dir: The working directory.
    :param show_flag: The "show" flag of the executed program:
        SW_HIDE = Hidden window
        SW_MINIMIZE = Minimized window
        SW_MAXIMIZE = Maximized window
    :return:
    """
    ret = AUTO_IT.AU3_RunAsWait(
        LPCWSTR(user), LPCWSTR(domain), LPCWSTR(password), INT(logon_flag),
        LPCWSTR(filename), LPCWSTR(work_dir), INT(show_flag)
    )
    return ret


@api.check(2, "set shutdown failed")
def shutdown(code):
    """

    :param code: The shutdown code is a combination of the following values:
        0 = Logoff
        1 = Shutdown
        2 = Reboot
        4 = Force
        8 = Power down
    :return:
    """
    ret = AUTO_IT.AU3_Shutdown(INT(code))
    return ret


# -*- coding: utf-8 -*-

__author__ = 'Jace Xu'

import os
import platform
from ctypes.wintypes import *
from functools import wraps
import sys
if sys.version_info[0] == 3:
    from functools import reduce
else:
    import ctypes

dll = "AutoItX3.dll"
bit, _ = platform.architecture()

if bit == "64bit":
    # if 64bit version of python within 64bit version of Windows,
    # load AutoItX3_x64.dll
    dll = "AutoItX3_x64.dll"

dll_path = os.path.join(os.path.dirname(__file__), "lib", dll)

if not os.path.exists(dll_path):
    raise IOError("Cannot load AutoItX from path: %s" % dll_path)

AUTO_IT = ctypes.windll.LoadLibrary(dll_path)


class AutoItError(Exception):
    pass


def error():
    return AUTO_IT.AU3_error()


class AutoItAPI(object):

    def __init__(self):
        self.msg = {}

    @staticmethod
    def _has_error():
        return True if error() == 1 else False

    @staticmethod
    def _has_unexpected_ret(ret, unexpected):
        if ret in unexpected:
            return True
        return False

    @staticmethod
    def _parser(x, y):
        if x["num"] >= y:
            x["flags"].append(y)
            x["num"] -= y
        return x

    def check(self, mark=0, err_msg="", **kwds):
        """
        :param mark:
            0 - do not need check return value or error()
            1 - check error()
            2 - check return value
        """
        unexpected_ret = kwds.get("unexpected_ret", (0,))

        def _check(fn):
            @wraps(fn)
            def wrapper(*args, **kwargs):
                ret = fn(*args, **kwargs)

                flags = reduce(
                    self._parser, [dict(num=mark, flags=[]), 2, 1])["flags"]

                if 1 in flags:
                    if self._has_error():
                        raise AutoItError(err_msg)

                if 2 in flags:
                    if self._has_unexpected_ret(ret, unexpected_ret):
                        raise AutoItError(err_msg)

                return ret
            return wrapper
        return _check


api = AutoItAPI()


@api.check()
def auto_it_set_option(option, param):
    """
    Changes the operation of various AutoIt functions/parameters
    :param option: The option to change
    :param param: The parameter (varies by option).
    :return:
    """
    pre_value = AUTO_IT.AU3_AutoItSetOption(LPCWSTR(option), INT(param))
    return pre_value


class Properties(object):
    """
    Below is an list of all the properties available in AutoItX.
    """
    SW_HIDE = 0
    SW_MAXIMIZE = 3
    SW_MINIMIZE = 6
    SW_RESTORE = 9
    SW_SHOW = 5
    SW_SHOWDEFAULT = 10
    SW_SHOWMAXIMIZED = 3
    SW_SHOWMINIMIZED = 2
    SW_SHOWMINNOACTIVE = 7
    SW_SHOWNA = 8
    SW_SHOWNOACTIVATE = 4
    SW_SHOWNORMAL = 1


class _Options(object):

    def __init__(self):
        self._caret_coord_mode = 1
        self._mouse_click_delay = 10
        self._mouse_click_down_delay = 10
        self._mouse_click_drag_delay = 250
        self._mouse_coord_mode = 1
        self._pixel_coord_mode = 1
        self._send_attach_mode = 0
        self._send_capslock_mode = 1
        self._send_key_delay = 5
        self._send_key_down_delay = 10
        self._win_detect_hidden_text = 0
        self._win_search_children = 0
        self._win_text_match_mode = 1
        self._win_title_match_mode = 1
        self._win_wait_delay = 250

    @property
    def caret_coord_mode(self):
        return self._caret_coord_mode

    @caret_coord_mode.setter
    def caret_coord_mode(self, value):
        auto_it_set_option("CaretCoordMode", value)
        self._caret_coord_mode = value

    @property
    def mouse_click_delay(self):
        return self._mouse_click_delay

    @mouse_click_delay.setter
    def mouse_click_delay(self, value):
        auto_it_set_option("MouseClickDelay", value)
        self._mouse_click_delay = value

    @property
    def mouse_click_down_delay(self):
        return self._mouse_click_down_delay

    @mouse_click_down_delay.setter
    def mouse_click_down_delay(self, value):
        auto_it_set_option("MouseClickDownDelay", value)
        self._mouse_click_down_delay = value

    @property
    def mouse_click_drag_delay(self):
        return self._mouse_click_drag_delay

    @mouse_click_drag_delay.setter
    def mouse_click_drag_delay(self, value):
        auto_it_set_option("MouseClickDragDelay", value)
        self._mouse_click_drag_delay = value

    @property
    def mouse_coord_mode(self):
        return self._mouse_coord_mode

    @mouse_coord_mode.setter
    def mouse_coord_mode(self, value):
        auto_it_set_option("MouseCoordMode", value)
        self._mouse_coord_mode = value

    @property
    def pixel_coord_mode(self):
        return self._pixel_coord_mode

    @pixel_coord_mode.setter
    def pixel_coord_mode(self, value):
        auto_it_set_option("PixelCoordMode", value)
        self._pixel_coord_mode = value

    @property
    def send_attach_mode(self):
        return self._send_attach_mode

    @send_attach_mode.setter
    def send_attach_mode(self, value):
        auto_it_set_option("SendAttachMode", INT(value))
        self._send_attach_mode = value

    @property
    def send_capslock_mode(self):
        return self._send_capslock_mode

    @send_capslock_mode.setter
    def send_capslock_mode(self, value):
        auto_it_set_option("SendCapslockMode", value)
        self._send_capslock_mode = value

    @property
    def send_key_delay(self):
        return self._send_key_delay

    @send_key_delay.setter
    def send_key_delay(self, value):
        auto_it_set_option("SendKeyDelay", value)
        self._send_key_delay = value

    @property
    def send_key_down_delay(self):
        return self._send_key_down_delay

    @send_key_down_delay.setter
    def send_key_down_delay(self, value):
        auto_it_set_option("SendKeyDownDelay", value)
        self._send_key_down_delay = value

    @property
    def win_detect_hidden_text(self):
        return self._win_detect_hidden_text

    @win_detect_hidden_text.setter
    def win_detect_hidden_text(self, value):
        auto_it_set_option("WinDetectHiddenText", value)
        self._win_detect_hidden_text = value

    @property
    def win_search_children(self):
        return self._win_search_children

    @win_search_children.setter
    def win_search_children(self, value):
        auto_it_set_option("WinSearchChildren", value)
        self._win_search_children = value

    @property
    def win_text_match_mode(self):
        return self._win_text_match_mode

    @win_text_match_mode.setter
    def win_text_match_mode(self, value):
        auto_it_set_option("WinTextMatchMode", value)
        self._win_text_match_mode = value

    @property
    def win_title_match_mode(self):
        return self._win_title_match_mode

    @win_title_match_mode.setter
    def win_title_match_mode(self, value):
        auto_it_set_option("WinTitleMatchMode", value)
        self._win_title_match_mode = value

    @property
    def win_wait_delay(self):
        return self._win_wait_delay

    @win_wait_delay.setter
    def win_wait_delay(self, value):
        auto_it_set_option("WinWaitDelay", value)
        self._win_wait_delay = value


class Commands(object):

    is_visible = "IsVisible"
    is_enabled = "IsEnabled"
    show_drop_down = "ShowDropDown"
    hide_drop_down = "HideDropDown"
    add_string = "AddString"
    del_string = "DelString"
    find_string = "FindString"
    set_current_selection = "SetCurrentSelection"
    is_checked = "IsChecked"
    check = "Check"
    un_check = "UnCheck"
    get_current_line = "GetCurrentLine"
    get_current_col = "GetCurrentCol"
    get_current_selection = "GetCurrentSelection"
    get_line_count = "GetLineCount"
    get_line = "GetLine"
    get_selected = "GetSelected"
    edit_paste = "EditPaste"
    current_tab = "CurrentTab"
    tab_right = "TabRight"
    tab_left = "TabLeft"
    de_select = "DeSelect"
    find_item = "FindItem"
    get_item_count = "GetItemCount"
    get_selected_count = "GetSelectedCount"
    get_sub_item_count = "GetSubItemCount"
    get_text = "GetText"
    is_selected = "IsSelected"
    select = "Select"
    select_all = "SelectAll"
    select_clear = "SelectClear"
    select_invert = "SelectInvert"
    view_change = "ViewChange"
    collapse = "Collapse"
    exists = "Exists"
    expand = "Expand"
    uncheck = "Uncheck"

options = _Options()
properties = Properties
commands = Commands
INTDEFAULT = -2147483647


@api.check(1, err_msg="clipboard is empty or contains a non-text entry")
def clip_get(buf_size=256):
    """

    :param buf_size:
    :return:
    """

    clip = ctypes.create_unicode_buffer(buf_size)
    AUTO_IT.AU3_ClipGet(clip, INT(buf_size))
    return clip.value.rstrip()


@api.check(2, err_msg="Write text to clipboard failed")
def clip_put(value):
    """

    :param value:
    :return:
    """
    ret = AUTO_IT.AU3_ClipPut(LPCWSTR(value))
    return ret


def is_admin():
    """

    :return:
    """
    ret = AUTO_IT.AU3_IsAdmin()
    return ret


def drive_map_add(device, share, flag=0, user="", pwd="", buf_size=256):
    """

    :param device:
    :param share:
    :param flag: 0 = default
        1 = Persistant mapping
        8 = Show authentication dialog if required
    :param user:
    :param pwd:
    :param buf_size:
    :return:
    """
    result = ctypes.create_unicode_buffer(buf_size)

    err_code = {
        1: "Undefined / Other error",
        2: "Access to the remote share was denied",
        3: "The device is already assigned",
        4: "Invalid device name",
        5: "Invalid remote share",
        6: "Invalid password"
    }
    AUTO_IT.AU3_DriveMapAdd(
        LPCWSTR(device), LPCWSTR(share), INT(flag), LPCWSTR(user),
        LPCWSTR(pwd), result, INT(buf_size))

    if error():
        raise AutoItError(err_code.get(error(), None))
    return result.value.rstrip()


@api.check(2, err_msg="the disconnection was unsuccessful")
def drive_map_del(device):
    """

    :param device:
    :return:
    """
    ret = AUTO_IT.AU3_DriveMapDel(LPCWSTR(device))
    return ret


@api.check(1, err_msg="get the details of a mapped drive failed")
def drive_map_get(device, buf_size=256):
    """

    :param device:
    :param buf_size:
    :return:
    """
    mapping = ctypes.create_unicode_buffer(buf_size)
    AUTO_IT.AU3_DriveMapGet(LPCWSTR(device), mapping, INT(buf_size))
    return mapping.value.rstrip()


def mouse_click(button="left", x=INTDEFAULT, y=INTDEFAULT, clicks=1, speed=-1):
    """

    :param button:
    :param x:
    :param y:
    :param clicks:
    :param speed:
    :return:
    """
    ret = AUTO_IT.AU3_MouseClick(
        LPCWSTR(button), INT(x), INT(y), INT(clicks), INT(speed)
    )
    return ret


def mouse_click_drag(x1, y1, x2, y2, button="left", speed=-1):
    """

    :param x1:
    :param y1:
    :param x2:
    :param y2:
    :param button:
    :param speed:
    :return:
    """

    ret = AUTO_IT.AU3_MouseClickDrag(
        LPCWSTR(button), INT(x1), INT(y1), INT(x2), INT(y2), INT(speed)
    )
    return ret


def mouse_down(button="left"):
    """

    :param button:
    :return:
    """
    AUTO_IT.AU3_MouseDown(LPCWSTR(button))


def mouse_get_cursor():
    """

    :return:
    """
    ret = AUTO_IT.AU3_MouseGetCursor()
    return ret


def mouse_get_pos():
    """

    :return:
    """
    p = POINT()
    AUTO_IT.AU3_MouseGetPos(ctypes.byref(p))
    return p.x, p.y


def mouse_move(x, y, speed=-1):
    """

    :param x:
    :param y:
    :param speed:
    :return:
    """
    ret = AUTO_IT.AU3_MouseMove(INT(x), INT(y), INT(speed))
    return ret


def mouse_up(button="left"):
    """

    :param button:
    :return:
    """
    AUTO_IT.AU3_MouseUp(LPCWSTR(button))


@api.check(1, err_msg="the direction is not recognized")
def mouse_wheel(direction, clicks=-1):
    """

    :param direction: "up" or "down"
    :param clicks:
    :return:
    """
    AUTO_IT.AU3_MouseWheel(LPCWSTR(direction), INT(clicks))


def opt(option, value):
    """

    :param option:
    :param value:
    :return:
    """
    return auto_it_set_option(option, value)


def pixel_checksum(left, top, right, bottom, step=1):
    """

    :param left:
    :param top:
    :param right:
    :param bottom:
    :param step:
    :return:
    """
    rect = RECT(left, top, right, bottom)
    ret = AUTO_IT.AU3_PixelChecksum(ctypes.byref(rect), INT(step))
    return ret


@api.check(2, unexpected_ret=(-1,), err_msg="invalid coordinates")
def pixel_get_color(x, y):
    """

    :param x:
    :param y:
    :return:
    """
    ret = AUTO_IT.AU3_PixelGetColor(INT(x), INT(y))
    return ret


@api.check(1, err_msg="color is not found")
def pixel_search(left, top, right, bottom, col, var=1, step=1):
    """

    :param left:
    :param top:
    :param right:
    :param bottom:
    :param col:
    :param var:
    :param step:
    :return:
    """
    p = POINT()
    rect = RECT(left, top, right, bottom)

    AUTO_IT.AU3_PixelSearch(
        ctypes.byref(rect), INT(col), INT(var), INT(step), ctypes.byref(p)
    )
    return p.x, p.y


def send(send_text, mode=0):
    """
    Sends simulated keystrokes to the active window.
    :param send_text:
    :param mode: Changes how "keys" is processed:
        flag = 0 (default), Text contains special characters like + and ! to
         indicate SHIFT and ALT key presses.
        flag = 1, keys are sent raw.
    :return:
    """
    AUTO_IT.AU3_Send(LPCWSTR(send_text), INT(mode))


def tooltip(tip, x=INTDEFAULT, y=INTDEFAULT):
    """

    :param tip:
    :param x:
    :param y:
    :return:
    """
    AUTO_IT.AU3_ToolTip(LPCWSTR(tip), INT(x), INT(y))



# -*- coding: utf-8 -*-

__author__ = 'Jace Xu'

from .autoit import AUTO_IT
from .autoit import api
from .autoit import AutoItError
from ctypes.wintypes import *
from ctypes import create_unicode_buffer, byref
from .autoit import properties


@api.check(2, "activate window failed, maybe the window doesn't exist")
def win_activate(title, **kwargs):
    """
    Activates (gives focus to) a window.
    :param title:
    :param text:
    :return:
    """
    text = kwargs.get("text", "")
    ret = AUTO_IT.AU3_WinActivate(LPCWSTR(title), LPCWSTR(text))
    return ret


@api.check(2, "activate window failed, maybe the window doesn't exist")
def win_activate_by_handle(handle):
    """

    :param handle:
    :return:
    """
    ret = AUTO_IT.AU3_WinActivateByHandle(HWND(handle))
    return ret


def win_active(title, **kwargs):
    """

    :param title:
    :param text:
    :return:
    """
    text = kwargs.get("text", "")
    ret = AUTO_IT.AU3_WinActive(LPCWSTR(title), LPCWSTR(text))
    return ret


def win_active_by_handle(handle):
    """

    :param handle:
    :return:
    """
    ret = AUTO_IT.AU3_WinActiveByHandle(HWND(handle))
    return ret


@api.check(2, "close this window failed, maybe the window doesn't exist")
def win_close(title, **kwargs):
    """

    :param title:
    :param text:
    :return:
    """
    text = kwargs.get("text", "")
    ret = AUTO_IT.AU3_WinClose(LPCWSTR(title), LPCWSTR(text))
    return ret


@api.check(2, "close window failed, maybe the window does not exist")
def win_close_by_handle(handle):
    """

    :param handle:
    :return:
    """
    ret = AUTO_IT.AU3_WinCloseByHandle(HWND(handle))
    return ret


def win_exists(title, **kwargs):
    """
    Checks to see if a specified window exists.
    :param title: The title of the window to check.
    :param text: The text of the window to check.
    :return: Returns 1 if the window exists, otherwise returns 0.
    """
    text = kwargs.get("text", "")
    ret = AUTO_IT.AU3_WinExists(LPCWSTR(title), LPCWSTR(text))
    return ret


def win_exists_by_handle(handle):
    """

    :param handle:
    :return:
    """
    ret = AUTO_IT.AU3_WinExistsByHandle(HWND(handle))
    return ret


@api.check(1, "get the coordinates of the caret failed")
def win_get_caret_pos():
    """
    Returns the coordinates of the caret in the foreground window
    :return:
    """
    p = POINT()
    AUTO_IT.AU3_WinGetCaretPos(byref(p))
    return p.x, p.y


def win_get_class_list(title, buf_size=200, **kwargs):
    """

    :param title:
    :param text:
    :param buf_size:
    :return:
    """
    text = kwargs.get("text", "")
    rec_text = create_unicode_buffer(buf_size)  # ...
    AUTO_IT.AU3_WinGetClassList(LPCWSTR(title), LPCWSTR(text),
                                rec_text, INT(buf_size))

    msg = rec_text.value.rstrip()
    return msg


def win_get_class_list_by_handle(handle, buf_size=200):
    """

    :param handle:
    :param buf_size:
    :return:
    """

    rec_text = create_unicode_buffer(buf_size)
    AUTO_IT.AU3_WinGetClassListByHandle(HWND(handle), rec_text, INT(buf_size))

    msg = rec_text.value.rstrip()
    return msg


def win_get_client_size(title, **kwargs):
    """

    :param title:
    :param text:
    :return:
    """
    text = kwargs.get("text", "")
    rect = RECT()

    ret = AUTO_IT.AU3_WinGetClientSize(LPCWSTR(title), LPCWSTR(text),
                                       byref(rect))
    if ret == 1:
        raise AutoItError(
            "get the size of client failed")
    return rect.right, rect.bottom


def win_get_client_size_by_handle(handle):
    """

    :param handle:
    :return:
    """
    rect = RECT()
    ret = AUTO_IT.AU3_WinGetClientSizeByHandle(HWND(handle), byref(rect))
    if ret == 1:
        raise AutoItError("get the size of client failed")
    return rect.right, rect.bottom


@api.check(3, "No window match the criteria")
def win_get_handle(title, **kwargs):
    """

    :param title:
    :param text:
    :return:
    """
    text = kwargs.get("text", "")
    ret = AUTO_IT.AU3_WinGetHandle(LPCWSTR(title), LPCWSTR(text))
    return ret


@api.check(1, "No window match the criteria")
def win_get_handle_as_text(title, buf_size=16, **kwargs):
    """

    :param title:
    :param text:
    :param buf_size:
    :return:
    """
    text = kwargs.get("text", "")
    rec_text = create_unicode_buffer(buf_size)
    AUTO_IT.AU3_WinGetHandleAsText(LPCWSTR(title), LPCWSTR(text),
                                   rec_text, INT(buf_size))
    msg = rec_text.value
    return msg


def win_get_pos(title, **kwargs):
    """

    :param title:
    :param text:
    :return:
    """
    text = kwargs.get("text", "")
    rect = RECT()
    res = AUTO_IT.AU3_WinGetPos(LPCWSTR(title), LPCWSTR(text), byref(rect))
    if res == 1:
        raise AutoItError("No window match the criteria")
    return rect.left, rect.top, rect.right, rect.bottom


def win_get_pos_by_handle(handle):
    """

    :param handle:
    :return:
    """
    rect = RECT()
    res = AUTO_IT.AU3_WinGetPosByHandle(HWND(handle), byref(rect))
    if res == 1:
        raise AutoItError("No window match the handle: %s" % str(handle))
    return rect.left, rect.top, rect.right, rect.bottom


@api.check(2, "No window match the criteria", unexpected_ret=(-1,))
def win_get_process(title, **kwargs):
    """

    :param title:
    :param text:
    :return:
    """
    text = kwargs.get("text", "")
    res = AUTO_IT.AU3_WinGetProcess(LPCWSTR(title), LPCWSTR(text))
    return res


@api.check(2, "No window match the criteria", unexpected_ret=(-1,))
def win_get_process_by_handle(handle):
    """

    :param handle:
    :return:
    """
    res = AUTO_IT.AU3_WinGetProcessByHandle(HWND(handle))
    return res


@api.check(2, "No window match the criteria")
def win_get_state(title, **kwargs):
    """
    Retrieves the state of a given window.
    :param title:
    :param text:
    :return:
    1 = Window exists
    2 = Window is visible
    4 = Windows is enabled
    8 = Window is active
    16 = Window is minimized
    32 = Windows is maximized
    """
    text = kwargs.get("text", "")
    res = AUTO_IT.AU3_WinGetState(LPCWSTR(title), LPCWSTR(text))
    return res


@api.check(2, "No window match the criteria")
def win_get_state_by_handle(handle):
    """

    :param handle:
    :return:
    """
    res = AUTO_IT.AU3_WinGetStateByHandle(HWND(handle))
    return res


def win_get_text(title, buf_size=256, **kwargs):
    """

    :param title:
    :param text:
    :param buf_size:
    :return:
    """
    text = kwargs.get("text", "")
    ret_text = create_unicode_buffer(buf_size)
    AUTO_IT.AU3_WinGetText(LPCWSTR(title), LPCWSTR(text), ret_text,
                           INT(buf_size))
    val = ret_text.value.rstrip()
    return val


def win_get_text_by_handle(handle, buf_size=256):
    """

    :param handle:
    :return:
    """
    ret_text = create_unicode_buffer(buf_size)
    AUTO_IT.AU3_WinGetTextByHandle(HWND(handle), ret_text, INT(buf_size))
    return ret_text.value.rstrip()


def win_get_title(title, buf_size=256, **kwargs):
    """

    :param title:
    :param text:
    :return:
    """
    text = kwargs.get("text", "")
    ret_text = create_unicode_buffer(buf_size)
    AUTO_IT.AU3_WinGetTitle(LPCWSTR(title), LPCWSTR(text), ret_text,
                            INT(buf_size))
    val = ret_text.value.rstrip()
    return val


def win_get_title_by_handle(handle, buf_size=256):
    """

    :param handle:
    :param buf_size:
    :return:
    """
    ret_text = create_unicode_buffer(buf_size)
    AUTO_IT.AU3_WinGetTitleByHandle(HWND(handle), ret_text, INT(buf_size))
    val = ret_text.value.rstrip()
    return val


@api.check(2, "No window match the criteria")
def win_kill(title, **kwargs):
    """

    :param title:
    :param text:
    :return:
    """
    text = kwargs.get("text", "")
    ret = AUTO_IT.AU3_WinKill(LPCWSTR(title), LPCWSTR(text))
    return ret


@api.check(2, "No window match the criteria")
def win_kill_by_handle(handle):
    """

    :param handle:
    :return:
    """
    ret = AUTO_IT.AU3_WinKillByHandle(HWND(handle))
    return ret


@api.check(2, "the menu item could not be found")
def win_menu_select_item(title, *items, **kwargs):
    """
    Usage:
        win_menu_select_item("[CLASS:Notepad]", "", u"文件(&F)", u"退出(&X)")
    :param title:
    :param text:
    :param items:
    :return:
    """
    text = kwargs.get("text", "")
    if not (0 < len(items) < 8):
        raise ValueError("accepted none item or number of items exceed eight")
    f_items = [LPCWSTR(item) for item in items]
    for i in range(8 - len(f_items)):
        f_items.append(LPCWSTR(""))

    ret = AUTO_IT.AU3_WinMenuSelectItem(LPCWSTR(title), LPCWSTR(text),
                                        *f_items)
    return ret


@api.check(2, "the menu item could not be found")
def win_menu_select_item_by_handle(handle, *items):
    """

    :param handle:
    :param items:
    :return:
    """
    if not (0 < len(items) < 8):
        raise ValueError("accepted none item or number of items exceed eight")
    f_items = [LPCWSTR(item) for item in items]
    for i in range(8 - len(f_items)):
        f_items.append(LPCWSTR(""))

    ret = AUTO_IT.AU3_WinMenuSelectItemByHandle(HWND(handle), *f_items)
    return ret


def win_minimize_all():
    """

    :return:
    """
    AUTO_IT.AU3_WinMinimizeAll()


def win_minimize_all_undo():
    """

    :return:
    """
    AUTO_IT.AU3_WinMinimizeAllUndo()


@api.check(2, "No window match the criteria")
def win_move(title, x, y, width=-1, height=-1, **kwargs):
    """

    :param title:
    :param text:
    :param x:
    :param y:
    :param width:
    :param height:
    :return:
    """
    text = kwargs.get("text", "")
    ret = AUTO_IT.AU3_WinMove(LPCWSTR(title), LPCWSTR(text), INT(x), INT(y),
                              INT(width), INT(height))
    return ret


@api.check(2, "No window match the criteria")
def win_move_by_handle(handle, x, y, width=-1, height=-1):
    """

    :param handle:
    :param x:
    :param y:
    :param width:
    :param height:
    :return:
    """
    ret = AUTO_IT.AU3_WinMoveByHandle(HWND(handle), INT(x), INT(y), INT(width),
                                      INT(height))
    return ret


@api.check(2, "No window match the criteria")
def win_set_on_top(title, flag=1, **kwargs):
    """

    :param title:
    :param flag: 1=set on top flag, 0 = remove on top flag
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")

    ret = AUTO_IT.AU3_WinSetOnTop(LPCWSTR(title), LPCWSTR(text), INT(flag))
    return ret


@api.check(2, "No window match the criteria")
def win_set_on_top_by_handle(handle, flag=1):
    """

    :param handle:
    :param flag:
    :return:
    """
    ret = AUTO_IT.AU3_WinSetOnTopByHandle(HWND(handle), INT(flag))
    return ret


@api.check(2, "No window match the criteria")
def win_set_state(title, flag=properties.SW_SHOW, **kwargs):
    """

    :param title:
    :param flag: The "show" flag of the executed program:
        SW_HIDE = Hide window
        SW_SHOW = Shows a previously hidden window
        SW_MINIMIZE = Minimize window
        SW_MAXIMIZE = Maximize window
        SW_RESTORE = Undoes a window minimization or maximization
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")

    ret = AUTO_IT.AU3_WinSetState(LPCWSTR(title), LPCWSTR(text), INT(flag))
    return ret


@api.check(2, "No window match the criteria")
def win_set_state_by_handle(handle, flag=properties.SW_SHOW):
    """

    :param handle:
    :param flag:
    :return:
    """
    ret = AUTO_IT.AU3_WinSetStateByHandle(HWND(handle), INT(flag))
    return ret


@api.check(2, "No window match the criteria")
def win_set_title(title, new_title, **kwargs):
    """

    :param title:
    :param new_title:
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")
    ret = AUTO_IT.AU3_WinSetTitle(LPCWSTR(title), LPCWSTR(text),
                                  LPCWSTR(new_title))
    return ret


@api.check(2, "No window match the criteria")
def win_set_title_by_handle(handle, new_title):
    """

    :param handle:
    :param new_title:
    :return:
    """
    ret = AUTO_IT.AU3_WinSetTitleByHandle(HWND(handle), LPCWSTR(new_title))
    return ret


@api.check(2, "No window match the criteria")
def win_set_trans(title, trans, **kwargs):
    """
    Sets the transparency of a window.
    :param title:
    :param trans: A number in the range 0 - 255. The larger the number,
        the more transparent the window will become.
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")

    ret = AUTO_IT.AU3_WinSetTrans(LPCWSTR(title), LPCWSTR(text), INT(trans))
    return ret


@api.check(2, "No window match the criteria")
def win_set_trans_by_handle(handle, trans):
    """

    :param handle:
    :param trans:
    :return:
    """
    ret = AUTO_IT.AU3_WinSetTransByHandle(HWND(handle), INT(trans))
    return ret


@api.check(2, "timeout on wait for window exists")
def win_wait(title, timeout=0, **kwargs):
    """

    :param title:
    :param timeout:
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")

    ret = AUTO_IT.AU3_WinWait(LPCWSTR(title), LPCWSTR(text), INT(timeout))
    return ret


@api.check(2, "timeout on wait for window exists")
def win_wait_by_handle(handle, timeout):
    """

    :param handle:
    :param timeout:
    :return:
    """
    ret = AUTO_IT.AU3_WinWaitByHandle(HWND(handle), INT(timeout))
    return ret


@api.check(2, "timeout on wait for activate window")
def win_wait_active(title, timeout=0, **kwargs):
    """

    :param title:
    :param timeout:
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")

    ret = AUTO_IT.AU3_WinWaitActive(LPCWSTR(title), LPCWSTR(text),
                                    INT(timeout))
    return ret


@api.check(2, "timeout on wait for activate window")
def win_wait_active_by_handle(handle, timeout):
    """

    :param handle:
    :param timeout:
    :return:
    """
    ret = AUTO_IT.AU3_WinWaitActiveByHandle(HWND(handle), INT(timeout))
    return ret


@api.check(2, "timeout on wait for close window")
def win_wait_close(title, timeout=0, **kwargs):
    """

    :param title:
    :param timeout:
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")
    ret = AUTO_IT.AU3_WinWaitClose(LPCWSTR(title), LPCWSTR(text), INT(timeout))
    return ret


@api.check(2, "timeout on wait for close window")
def win_wait_close_by_handle(handle, timeout):
    """

    :param handle:
    :param timeout:
    :return:
    """
    ret = AUTO_IT.AU3_WinWaitCloseByHandle(HWND(handle), INT(timeout))
    return ret


@api.check(2, "timeout on wait for deactivate window")
def win_wait_not_active(title, timeout=0, **kwargs):
    """

    :param title:
    :param timeout:
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")

    ret = AUTO_IT.AU3_WinWaitNotActive(LPCWSTR(title), LPCWSTR(text),
                                       INT(timeout))
    return ret


@api.check(2, "timeout on wait for deactivate window")
def win_wait_not_active_by_handle(handle, timeout):
    """

    :param handle:
    :param timeout:
    :return:
    """
    ret = AUTO_IT.AU3_WinWaitNotActiveByHandle(HWND(handle), INT(timeout))
    return ret



# -*- coding: utf-8 -*-

__author__ = 'Jace Xu'
__version__ = "0.5"

from .autoit import options, properties, commands
from .autoit import AutoItError

from .autoit import error
from .autoit import auto_it_set_option
from .autoit import clip_get
from .autoit import clip_put
from .autoit import is_admin
from .autoit import drive_map_add
from .autoit import drive_map_del
from .autoit import drive_map_get
from .autoit import mouse_click
from .autoit import mouse_click_drag
from .autoit import mouse_down
from .autoit import mouse_get_cursor
from .autoit import mouse_get_pos
from .autoit import mouse_move
from .autoit import mouse_up
from .autoit import mouse_wheel
from .autoit import opt
from .autoit import pixel_checksum
from .autoit import pixel_get_color
from .autoit import pixel_search
from .autoit import send
from .autoit import tooltip

from .process import run
from .process import run_wait
from .process import process_close
from .process import process_exists
from .process import process_set_priority
from .process import process_wait
from .process import process_wait_close
from .process import run_as
from .process import run_as_wait
from .process import shutdown

from .win import win_activate
from .win import win_activate_by_handle
from .win import win_active
from .win import win_active_by_handle
from .win import win_close
from .win import win_close_by_handle
from .win import win_exists
from .win import win_exists_by_handle
from .win import win_get_caret_pos
from .win import win_get_class_list
from .win import win_get_class_list_by_handle
from .win import win_get_client_size
from .win import win_get_client_size_by_handle
from .win import win_get_handle
from .win import win_get_handle_as_text
from .win import win_get_pos
from .win import win_get_pos_by_handle
from .win import win_get_process
from .win import win_get_process_by_handle
from .win import win_get_state
from .win import win_get_state_by_handle
from .win import win_get_text
from .win import win_get_text_by_handle
from .win import win_get_title
from .win import win_get_title_by_handle
from .win import win_kill
from .win import win_kill_by_handle
from .win import win_menu_select_item
from .win import win_menu_select_item_by_handle
from .win import win_minimize_all
from .win import win_minimize_all_undo
from .win import win_move
from .win import win_move_by_handle
from .win import win_set_on_top
from .win import win_set_on_top_by_handle
from .win import win_set_state
from .win import win_set_state_by_handle
from .win import win_set_title
from .win import win_set_title_by_handle
from .win import win_set_trans
from .win import win_set_trans_by_handle
from .win import win_wait
from .win import win_wait_by_handle
from .win import win_wait_active
from .win import win_wait_active_by_handle
from .win import win_wait_close
from .win import win_wait_close_by_handle
from .win import win_wait_not_active
from .win import win_wait_not_active_by_handle

from .control import control_click
from .control import control_click_by_handle
from .control import control_command
from .control import control_command_by_handle
from .control import control_list_view
from .control import control_list_view_by_handle
from .control import control_disable
from .control import control_disable_by_handle
from .control import control_enable
from .control import control_enable_by_handle
from .control import control_focus
from .control import control_focus_by_handle
from .control import control_get_focus
from .control import control_get_focus_by_handle
from .control import control_get_handle
from .control import control_get_handle_as_text
from .control import control_get_pos
from .control import control_get_pos_by_handle
from .control import control_get_text
from .control import control_get_text_by_handle
from .control import control_hide
from .control import control_hide_by_handle
from .control import control_move
from .control import control_move_by_handle
from .control import control_send
from .control import control_send_by_handle
from .control import control_set_text
from .control import control_set_text_by_handle
from .control import control_show
from .control import control_show_by_handle
from .control import control_tree_view
from .control import control_tree_view_by_handle
from .control import statusbar_get_text
from .control import statusbar_get_text_by_handle



# -*- coding: utf-8 -*-

__author__ = 'Jace Xu'

from .autoit import INTDEFAULT, AUTO_IT
from .autoit import api
from ctypes.wintypes import *
import ctypes


@api.check(2, "send click message failed")
def control_click(title, control, **kwargs):
    """

    :param title:
    :param text:
    :param control:
    :param button:
    :param clicks:
    :param x:
    :param y:
    :return:
    """
    text = kwargs.get("text", "")
    button = kwargs.get("button", "left")
    clicks = kwargs.get("clicks", 1)
    x = kwargs.get("x", INTDEFAULT)
    y = kwargs.get("y", INTDEFAULT)

    ret = AUTO_IT.AU3_ControlClick(LPCWSTR(title), LPCWSTR(text),
                                   LPCWSTR(control), LPCWSTR(button),
                                   INT(clicks), INT(x), INT(y))
    return ret


@api.check(2, "send click message failed")
def control_click_by_handle(hwnd, h_ctrl, **kwargs):
    """

    :param handle:
    :param kwargs:
    :return:
    """
    button = kwargs.get("button", "left")
    clicks = kwargs.get("clicks", 1)
    x = kwargs.get("x", INTDEFAULT)
    y = kwargs.get("y", INTDEFAULT)

    ret = AUTO_IT.AU3_ControlClickByHandle(HWND(hwnd), HWND(h_ctrl),
                                           LPCWSTR(button), INT(clicks),
                                           INT(x), INT(y))
    return ret


@api.check(1, "no window match the criteria")
def control_command(title, control, command, buf_size=256, **kwargs):
    """

    :param title:
    :param control:
    :param command:
    :param extra:
    :param buf_size:
    :return:
    """
    text = kwargs.get("text", "")
    extra = kwargs.get("extra", "")
    result = ctypes.create_unicode_buffer(buf_size)
    AUTO_IT.AU3_ControlCommand(LPCWSTR(title), LPCWSTR(text), LPCWSTR(control),
                               LPCWSTR(command), LPCWSTR(extra),
                               result, INT(buf_size))

    return result.value.rstrip()


@api.check(1, "no window match the criteria")
def control_command_by_handle(hwnd, h_ctrl, command, buf_size=256, **kwargs):
    """

    :param hwnd:
    :param h_ctrl:
    :param command:
    :param kwargs:
    :return:
    """
    extra = kwargs.get("extra", "")
    result = ctypes.create_unicode_buffer(buf_size)

    AUTO_IT.AU3_ControlCommandByHandle(
        HWND(hwnd), HWND(h_ctrl), LPCWSTR(command), LPCWSTR(extra), result,
        INT(buf_size))
    return result.value.rstrip()


@api.check(1, "Window/Control could not be found")
def control_list_view(title, control, command, **kwargs):
    """

    :param title:
    :param control:
    :param command:
    :param args:
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")
    buf_size = kwargs.get("buf_size", 256)
    result = ctypes.create_unicode_buffer(buf_size)
    extra1 = kwargs.get("extras1", "")
    extra2 = kwargs.get("extras2", "")

    AUTO_IT.AU3_ControlListView(
        LPCWSTR(title), LPCWSTR(text), LPCWSTR(control), LPCWSTR(command),
        LPCWSTR(extra1), LPCWSTR(extra2), result, INT(buf_size)
    )
    return result.value.rstrip()


@api.check(1, "Window/Control could not be found")
def control_list_view_by_handle(hwnd, h_ctrl, command, **kwargs):
    """

    :param hwnd:
    :param h_ctrl:
    :param command:
    :param kwargs:
    :return:
    """
    extra1 = kwargs.get("extra1", "")
    extra2 = kwargs.get("extra2", "")
    buf_size = kwargs.get("buf_size", 256)
    result = ctypes.create_unicode_buffer(buf_size)

    AUTO_IT.AU3_ControlListViewByHandle(
        HWND(hwnd), HWND(h_ctrl), LPCWSTR(command),
        LPCWSTR(extra1), LPCWSTR(extra2), result, INT(buf_size)
    )
    return result.value.rstrip()


@api.check(2, "Window/Control could not be found")
def control_disable(title, control, **kwargs):
    """

    :param title:
    :param control:
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")

    ret = AUTO_IT.AU3_ControlDisable(LPCWSTR(title), LPCWSTR(text),
                                     LPCWSTR(control))
    return ret


@api.check(2, "Window/Control could not be found")
def control_disable_by_handle(hwnd, h_ctrl):
    """

    :param hwnd:
    :param h_ctrl:
    :return:
    """
    ret = AUTO_IT.AU3_ControlDisableByHandle(HWND(hwnd), HWND(h_ctrl))
    return ret


@api.check(2, "Window/Control could not be found")
def control_enable(title, control, **kwargs):
    """

    :param title:
    :param control:
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")

    ret = AUTO_IT.AU3_ControlEnable(LPCWSTR(title), LPCWSTR(text),
                                    LPCWSTR(control))
    return ret


@api.check(2, "Window/Control could not be found")
def control_enable_by_handle(hwnd, h_ctrl):
    """

    :param hwnd:
    :param h_ctrl:
    :return:
    """
    ret = AUTO_IT.AU3_ControlEnableByHandle(HWND(hwnd), HWND(h_ctrl))
    return ret


@api.check(2, "Window/Control could not be found")
def control_focus(title, control, **kwargs):
    """

    :param title:
    :param control:
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")

    ret = AUTO_IT.AU3_ControlFocus(
        LPCWSTR(title), LPCWSTR(text), LPCWSTR(control))
    return ret


@api.check(2, "Window/Control could not be found")
def control_focus_by_handle(hwnd, h_ctrl):
    """

    :param hwnd:
    :param h_ctrl:
    :return:
    """
    ret = AUTO_IT.AU3_ControlFocusByHandle(HWND(hwnd), HWND(h_ctrl))
    return ret


@api.check(1, "Window/Control could not be found")
def control_get_focus(title, **kwargs):
    """

    :param title:
    :param kwargs:
    :return:
    """
    buf_size = kwargs.get("buf_size", 256)
    text = kwargs.get("text", "")
    ctrl_with_focus = ctypes.create_unicode_buffer(buf_size)

    AUTO_IT.AU3_ControlGetFocus(
        LPCWSTR(title), LPCWSTR(text), ctrl_with_focus, INT(buf_size))
    return ctrl_with_focus.value.rstrip()


@api.check(1, "Window/Control could not be found")
def control_get_focus_by_handle(hwnd, buf_size=256):
    """

    :param hwnd:
    :param buf_size:
    :return:
    """
    ctrl_with_focus = ctypes.create_unicode_buffer(buf_size)

    AUTO_IT.AU3_ControlGetFocusByHandle(HWND(hwnd), ctrl_with_focus,
                                        INT(buf_size))
    return ctrl_with_focus.value.rstrip()


@api.check(1, "Window/Control could not be found")
def control_get_handle(hwnd, control):
    """

    :param hwnd:
    :param control:
    :return:
    """
    ret = AUTO_IT.AU3_ControlGetHandle(HWND(hwnd), LPCWSTR(control))
    return ret


@api.check(1, "Window/Control could not be found")
def control_get_handle_as_text(title, control, **kwargs):
    """

    :param title:
    :param control:
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")
    buf_size = kwargs.get("buf_size", 32)
    ret_text = ctypes.create_unicode_buffer(buf_size)

    AUTO_IT.AU3_ControlGetHandleAsText(
        LPCWSTR(title), LPCWSTR(text), LPCWSTR(control),
        ret_text, INT(buf_size)
    )
    return ret_text.value.rstrip()


@api.check(1, "Window/Control could not be found")
def control_get_pos(title, control, text=""):
    """

    :param title:
    :param control:
    :param text:
    :return:
    """
    rect = RECT()

    AUTO_IT.AU3_ControlGetPos(
        LPCWSTR(title), LPCWSTR(text), LPCWSTR(control),
        ctypes.byref(rect)
    )
    return rect.left, rect.top, rect.right, rect.bottom


@api.check(1, "Window/Control could not be found")
def control_get_pos_by_handle(hwnd, h_ctrl):
    """

    :param hwnd:
    :param h_ctrl:
    :return:
    """
    rect = RECT()

    AUTO_IT.AU3_ControlGetPosByHandle(HWND(hwnd), HWND(h_ctrl),
                                      ctypes.byref(rect))
    return rect.left, rect.top, rect.right, rect.bottom


@api.check(1, "Window/Control could not be found")
def control_get_text(title, control, **kwargs):
    """

    :param title:
    :param control:
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")
    buf_size = kwargs.get("buf_size", 256)
    ctrl_text = ctypes.create_unicode_buffer(buf_size)

    AUTO_IT.AU3_ControlGetText(
        LPCWSTR(title), LPCWSTR(text), LPCWSTR(control),
        ctrl_text, INT(buf_size)
    )

    return ctrl_text.value.rstrip()


@api.check(1, "Window/Control could not be found")
def control_get_text_by_handle(hwnd, h_ctrl, **kwargs):
    """

    :param hwnd:
    :param h_ctrl:
    :return:
    """
    buf_size = kwargs.get("buf_size", 256)
    ctrl_text = ctypes.create_unicode_buffer(buf_size)

    AUTO_IT.AU3_ControlGetTextByHandle(
        HWND(hwnd), HWND(h_ctrl), ctrl_text, INT(buf_size)
    )

    return ctrl_text.value.rstrip()


@api.check(2, "Window/Control could not be found")
def control_hide(title, control, **kwargs):
    """

    :param title:
    :param control:
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")

    ret = AUTO_IT.AU3_ControlHide(
        LPCWSTR(title), LPCWSTR(text), LPCWSTR(control))
    return ret


@api.check(2, "Window/Control could not be found")
def control_hide_by_handle(hwnd, h_ctrl):
    """

    :param hwnd:
    :param h_ctrl:
    :return:
    """
    ret = AUTO_IT.AU3_ControlHideByHandle(HWND(hwnd), HWND(h_ctrl))
    return ret


@api.check(2, "Window/Control could not be found")
def control_move(title, control, x, y, width=-1, height=-1, **kwargs):
    """

    :param title:
    :param control:
    :param x:
    :param y:
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")

    ret = AUTO_IT.AU3_ControlMove(
        LPCWSTR(title), LPCWSTR(text), LPCWSTR(control),
        INT(x), INT(y), INT(width), INT(height)
    )
    return ret


@api.check(2, "Window/Control could not be found")
def control_move_by_handle(hwnd, h_ctrl, x, y, width=-1, height=-1):
    """

    :param hwnd:
    :param h_ctrl:
    :param x:
    :param y:
    :param width:
    :param height:
    :return:
    """
    ret = AUTO_IT.AU3_ControlMoveByHandle(
        HWND(hwnd), HWND(h_ctrl), INT(x), INT(y), INT(width), INT(height)
    )
    return ret


@api.check(2, "Window/Control could not be found")
def control_send(title, control, send_text, mode=0, **kwargs):
    """

    :param title:
    :param control:
    :param send_text:
    :param mode:
    flag = 0 (default), Text contains special characters like + to indicate
        SHIFT and {LEFT} to indicate left arrow.
    flag = 1, keys are sent raw.
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")

    ret = AUTO_IT.AU3_ControlSend(
        LPCWSTR(title), LPCWSTR(text), LPCWSTR(control),
        LPCWSTR(send_text), INT(mode)
    )
    return ret


@api.check(2, "Window/Control could not be found")
def control_send_by_handle(hwnd, h_ctrl, send_text, mode=0):
    """

    :param hwnd:
    :param h_ctrl:
    :param send_text:
    :param mode:
    :return:
    """

    ret = AUTO_IT.AU3_ControlSendByHandle(
        HWND(hwnd), HWND(h_ctrl), LPCWSTR(send_text), INT(mode)
    )
    return ret


@api.check(2, "Window/Control could not be found")
def control_set_text(title, control, control_text, **kwargs):
    """

    :param title:
    :param control:
    :param control_text:
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")

    ret = AUTO_IT.AU3_ControlSetText(
        LPCWSTR(title), LPCWSTR(text), LPCWSTR(control), LPCWSTR(control_text)
    )
    return ret


@api.check(2, "Window/Control could not be found")
def control_set_text_by_handle(hwnd, h_ctrl, control_text):
    """

    :param hwnd:
    :param h_ctrl:
    :param control_text:
    :return:
    """
    ret = AUTO_IT.AU3_ControlSetTextByHandle(
        HWND(hwnd), HWND(h_ctrl), LPCWSTR(control_text)
    )
    return ret


@api.check(2, "Window/Control could not be found")
def control_show(title, control, **kwargs):
    """

    :param title:
    :param control:
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")

    ret = AUTO_IT.AU3_ControlShow(
        LPCWSTR(title), LPCWSTR(text), LPCWSTR(control))
    return ret


@api.check(2, "Window/Control could not be found")
def control_show_by_handle(hwnd, h_ctrl):
    """

    :param hwnd:
    :param h_ctrl:
    :return:
    """
    ret = AUTO_IT.AU3_ControlShowByHandle(HWND(hwnd), HWND(h_ctrl))
    return ret


@api.check(1, "Window/Control could not be found")
def control_tree_view(title, control, command, **kwargs):
    """

    :param title:
    :param control:
    :param command:
    :param args:
    :param kwargs:
    :return:
    """
    text = kwargs.get("text", "")
    buf_size = kwargs.get("buf_size", 256)
    result = ctypes.create_unicode_buffer(buf_size)
    extra1 = kwargs.get("extras1", "")
    extra2 = kwargs.get("extras2", "")

    AUTO_IT.AU3_ControlTreeView(
        LPCWSTR(title), LPCWSTR(text), LPCWSTR(control), LPCWSTR(command),
        LPCWSTR(extra1), LPCWSTR(extra2), result, INT(buf_size)
    )

    return result.value.rstrip()


@api.check(1, "Window/Control could not be found")
def control_tree_view_by_handle(hwnd, h_ctrl, command, **kwargs):
    """

    :param hwnd:
    :param h_ctrl:
    :param command:
    :param kwargs:
    :return:
    """
    extra1 = kwargs.get("extra1", "")
    extra2 = kwargs.get("extra2", "")
    buf_size = kwargs.get("buf_size", 256)
    result = ctypes.create_unicode_buffer(buf_size)

    AUTO_IT.AU3_ControlTreeViewByHandle(
        HWND(hwnd), HWND(h_ctrl), LPCWSTR(command),
        LPCWSTR(extra1), LPCWSTR(extra2), result, INT(buf_size)
    )
    return result.value.rstrip()


@api.check(1, "Window/Control could not be found")
def statusbar_get_text(title, text="", part=1, buf_size=256):
    """

    :param title:
    :param text:
    :param part: The "part" number of the status bar to read - the default
        is 1. 1 is the first possible part and usually the one that contains
        the useful messages like "Ready" "Loading...", etc.
    :param buf_size:
    :return:
    """
    sb_text = ctypes.create_unicode_buffer(buf_size)

    AUTO_IT.AU3_StatusbarGetText(
        LPCWSTR(title), LPCWSTR(text), INT(part), sb_text, INT(buf_size)
    )

    return sb_text.value.rstrip()


@api.check(1, "Window/Control could not be found")
def statusbar_get_text_by_handle(hwnd, part=1, buf_size=256):
    """

    :param hwnd:
    :param part:
    :param buf_size:
    :return:
    """
    statusbar_text = ctypes.create_unicode_buffer(buf_size)

    AUTO_IT.AU3_StatusbarGetTextByHandle(
        HWND(hwnd), INT(part), statusbar_text, INT(buf_size)
    )

    return statusbar_text.value.rstrip()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import autoit
import os

setup(
    name='pyautoit-win64',
    version="1.0.3",
    packages=['autoit'],
    package_data={'': ['lib\\*.dll']},
    url='https://github.com/xuepl/pyautoit-64.git',
    license='MIT',
    author='xuepl',
    author_email='xuepl@guoyasoft.com',
    description='Python binding for AutoItX3.dll 支持64位dll',
    platforms="python"
)

